
page	60,132
title	Parkal - Park hard disk(s) at last usable cylinder (reptd by int 13h)

;----------------------------------------------------------------------------
; copyright = 1985, skip gilbrech
;
; author = Skip Gilbrech
;	   90 Lexington Ave. #10-G
;	   New York, NY 10016
;	   212-685-0551
;	   Compuserve 71445,534
;
; This program uses the int 13h (ah=8) call documented in the IBM PC Tech.
; Ref. Manual Fixed Disk Bios listing to determine the number of hard
; drives installed, and the highest usable cylinder on each.  It then
; attempts to move the heads to the highest cylinder, in preparation for
; moving or powering down the computer.  Since it uses the bios exclusively,
; it should be independent of any particular controller/rom combination, as
; long as the hard-disk is "xt-compatible".
;
; The assembled program, and this source, are released to the public domain
; with the understanding that they will not be used in any commercial or
; profit-making venture without the express permission of the author.
;
; date written = 10/03/85
;
; change activity =
;   
;   10/15/85 sg ny r1.0:	Sign-on message and version number added.
;
;   10/11/85 sg ny r0.4:	Zero all registers on initialization to cure
;				 problem noticed with Jim Gainsley's system
;				 where program ran fine under debug, but gave
;				 erroneous results when loaded by dos.
;
;   10/09/85 sg ny r0.3:	Wait for key to be hit before returning to
;				 dos to prevent possible head movement when
;				 prompt is redisplayed, etc.
;
;   10/07/85 sg ny r0.3:	Print out cyl, head, & sector where parked.
;
;   10/05/85 sg ny r0.2:	Fixed bug caused by seeking to sector 0
;				 which produced an error with some roms,
;				 not with others.  Program now seeks to
;				 highest usable cyl., head, & sector passed
;				 to it for each drive.
;
;   10/03/85 sg ny r0.1:	original date
;
; environment =
;   system = ibm pc (ms-dos)
;   processor = microsoft 8086 macro assembler
;
;----------------------------------------------------------------------------

VERSION	macro
	db	"1.0 - 10/15/85"
    endm

FALSE		equ	0
TRUE		equ	NOT FALSE

CR		equ	0DH
LF		equ	0AH

DOSINT		equ	21H
PRINT_STRING	equ	09H

DISKINT		equ	13H		; bios disk interrupt
GETPARMS	equ	08H		; return current drive parms
SEEK		equ	0CH		; seek to head,cyl,sector
RESET		equ	00H		; reset disk

DRIVE_1		equ	80H		; code for 1st fixed disk
DRIVE_2		equ	81H		; code for 2nd fixed disk
CYL_MASK	equ	11000000b	; retain high 2 bits of cyl. in CL

    page
;----------------------------------------------------------------------------

CODE	segment
assume	cs:CODE,ds:CODE

public START,NUMDRIVES,NOFD_MSG,FD1_MSG,FD2_MSG,PK_OK_MSG,PK_ERR_MSG,CYL_MSG
public CYL_BUF,CYL_BUF_END,HEAD_MSG,HEAD_BUF,HEAD_BUF_END,SEC_MSG,SEC_BUF,SEC_BUF_END
public PARKAL,PKA_1,PKA_2,PKA_EXIT,DO_PARK,DP_LOOP,DP_EXIT,DP_RESET,DPR_1,MSG
public MSG_1,MSG_2,MSG_3,MSG_EXIT,DO_MSG,FMT_CYL_NUM,FMT_HEAD_NUM,FMT_SEC_NUM
public FMT_DECIMAL,FD_LOOP,FD_EXIT

	org	100H			; for .com file
START:					; program entry point
	jmp	PARKAL

NUMDRIVES	db	?		; number of hard drives

SIGN_ON		db	"Parkal (V"
		VERSION
		db	") Copyright(C) 1985 Skip Gilbrech",CR,LF
		db	"For Public-Domain use -- not to be sold",CR,LF,"$"

NOFD_MSG	db	"No fixed disks installed",CR,LF,"$"
FD1_MSG		db	"1st Fixed disk: $"
FD2_MSG		db	"2nd Fixed disk: $"
PK_OK_MSG	db	"parked$"
PK_ERR_MSG	db	"error parking$"
CYL_MSG		db	" at cylinder $"
CYL_BUF		db	"    "
CYL_BUF_END	db	"$"
HEAD_MSG	db	", head $"
HEAD_BUF	db	" "
HEAD_BUF_END	db	"$"
SEC_MSG		db	", sector $"
SEC_BUF		db	"  "
CRLF_MSG	label byte
SEC_BUF_END	db	CR,LF,"$"
HIT_KEY_MSG	db	"Hit any key to return to dos...$"
    page
;----------------------------------------------------------------------------

PARKAL	proc near

	mov	dx,offset SIGN_ON	; print sign-on message
	call	DO_MSG			;

	sub	ax,ax			; zero all registers to avoid problems
	mov	bx,ax			;  (the results returned by some roms
	mov	cx,ax			;  seem to be affected by register
	mov	dx,ax			;  settings not documented in the
	mov	si,ax			;  ibm tech. ref. manual)
	mov	di,ax
	mov	bp,ax

	mov	ah,GETPARMS		; get drive parms
	mov	dl,DRIVE_1		; 1st drive
	int	DISKINT			;
	jc	PKA_1			; carry means no fixed disk, etc.
	or	dl,dl			; any drives?
	jnz	PKA_2			; yes, continue
PKA_1:
	sub	dl,dl			; zero DL
	call	MSG			; print no drives msg
	jmp short PKA_EXIT_1		; & exit
PKA_2:
	mov	NUMDRIVES,dl		; save number of drives

	mov	dl,DRIVE_1		; 1st drive
	call	DO_PARK			; park it

	cmp	NUMDRIVES,1		; more than one drive?
	je	PKA_EXIT		; no, we're done

	mov	ah,GETPARMS		; get drive parms
	mov	dl,DRIVE_2		; 2nd drive
	int	DISKINT			;

	mov	dl,DRIVE_2		; 2nd drive
	call	DO_PARK			; park it
PKA_EXIT:
	mov	dx,offset HIT_KEY_MSG	;
	call	DO_MSG			;
	call	GET_KEY			;
	mov	dx,offset CRLF_MSG	;
	call	DO_MSG			;
PKA_EXIT_1:				; (no prompt to hit key)
	int	20H			; exit to dos

PARKAL	endp

    page
;----------------------------------------------------------------------------
; Attempt to park the heads at cylinder/sector in CX (cylinder number low bits
; are in CH, high 2 bits are high 2 bits of CL -- sector number is low 5 bits
; of CL).  Drive number in DL, head number in DH.  Make 3 retries if error.

DO_PARK	proc near

	mov	bl,3			; retry count
DP_LOOP:
	push	bx			; save retry count
	push	cx			; save cyl/sector number
	push	dx			; save drive/head number
	mov	ah,SEEK			; seek to proper place
	int	DISKINT			;
	pop	dx			; restore drive/head number
	pop	cx			; restore cyl/sector number
	pop	bx			; restore retry count
	jc	DP_RESET		; carry = unsuccessful seek

	mov	bx,offset PK_OK_MSG	; point BX to ok message
DP_EXIT:
	call	MSG			;
	ret

DP_RESET:
	dec	bl			; decrement retry count
	jnz	DPR_1			; continue if more retries left
	mov	bx,offset PK_ERR_MSG	; point BX to error message
	jmp	DP_EXIT			; print msg & return
DPR_1:
	push	bx			; save retry count
	push	cx			; save cyl/sector number
	push	dx			; save drive/head number
	mov	ah,RESET		; reset disk
	int	DISKINT			;
	pop	dx			; restore drive/head number
	pop	cx			; restore cyl/sector number
	pop	bx			; restore retry count
	jmp	DP_LOOP			; & try again

DO_PARK	endp

    page
;----------------------------------------------------------------------------
; Print message pointed to by BX, referring to cylinder number in CX, after
; drive msg if DL isn't zero -- drive message determined by value in DL:
; 80H = 1st drive, 81H = 2nd drive.  If DL is zero, print 'no fixed disks'
; message.  Registers unchanged.

MSG	proc near

	push	ax
	push	bx
	push	dx

	or	dl,dl			; DL = 0?
	jnz	MSG_1			; no, continue
	mov	dx,offset NOFD_MSG	; else, print no drives msg
	call	DO_MSG			;
	jmp short MSG_EXIT		;
MSG_1:
	push	dx			; save drive/head number
	cmp	dl,DRIVE_1		; 1st drive?
	jne	MSG_2			; no, continue
	mov	dx,offset FD1_MSG	; else, point to 1st disk msg
	jmp short MSG_3			;
MSG_2:
	mov	dx,offset FD2_MSG	; point to 2nd disk msg
MSG_3:
	call	DO_MSG			;
	mov	dx,bx			; point DX to rest of message
	call	DO_MSG			;

	mov	dx,offset CYL_MSG	; print " at cylinder " message
	call	DO_MSG			;
	call	FMT_CYL_NUM		; returns ptr to start of number in DX
	call	DO_MSG			;

	mov	dx,offset HEAD_MSG	; print ", head " message
	call	DO_MSG			;
	pop	dx			; restore head number
	call	FMT_HEAD_NUM		; returns ptr to start of number in DX
	call	DO_MSG			;

	mov	dx,offset SEC_MSG	; print ", sector " message
	call	DO_MSG			;
	call	FMT_SEC_NUM		; returns ptr to start of number in DX
	call	DO_MSG			;
MSG_EXIT:
	pop	dx
	pop	bx
	pop	ax
	ret

DO_MSG	proc near			; print msg pointed to by DX

	mov	ah,PRINT_STRING
	int	DOSINT
	ret

DO_MSG	endp

MSG	endp

    page
;----------------------------------------------------------------------------
; Format the cylinder number (in CX) in decimal & place in CYL_BUF, returning
; pointer to beginning of number in DX.  Only DX changed.

FMT_CYL_NUM	proc near

	push	ax
	push	bx

; get actual cylinder number in AX

	mov	al,ch			; get lower 8 bits into AL
	mov	ah,cl			; get high 2 bits into high 2 bits of AH
	and	ah,CYL_MASK		; discard other bits
	rol	ah,1			; move the 2 bits into low 2 bits of AH
	rol	ah,1			;

	mov	bx,offset CYL_BUF_END-1	; point BX to end of the buffer
	call	FMT_DECIMAL		; pointer to start returned in DX

	pop	bx
	pop	ax
	ret

FMT_CYL_NUM	endp

    page
;----------------------------------------------------------------------------
; Format the head number (in DH) in decimal & place in HEAD_BUF, returning
; pointer to beginning of number in DX.  Only DX changed.

FMT_HEAD_NUM	proc near

	push	ax
	push	bx

	sub	ah,ah			; zero high byte of AX
	mov	al,dh			; get head number into AL

	mov	bx,offset HEAD_BUF_END-1	; point BX to end of the buffer
	call	FMT_DECIMAL		; pointer to start returned in DX

	pop	bx
	pop	ax
	ret

FMT_HEAD_NUM	endp

    page
;----------------------------------------------------------------------------
; Format the sector number (in CL) in decimal & place in SEC_BUF, returning
; pointer to beginning of number in DX.  Only DX changed.

FMT_SEC_NUM	proc near

	push	ax
	push	bx

	sub	ah,ah			; zero high byte of AX
	mov	al,cl			; get sector number into AL
	and	al,(not CYL_MASK)	; shut off high 2 bits of cyl. num.

	mov	bx,offset SEC_BUF_END-1	; point BX to end of the buffer
	call	FMT_DECIMAL		; pointer to start returned in DX

	pop	bx
	pop	ax
	ret

FMT_SEC_NUM	endp

    page
;----------------------------------------------------------------------------
; Format the number in AX in decimal & place in the buffer ending at BX,
; returning pointer to beginning of number in DX.  Only DX changed.

FMT_DECIMAL	proc near

	push	ax
	push	bx
	push	cx

	mov	cx,10			; put divisor in CX
FD_LOOP:
	sub	dx,dx			; zero high word of dividend
	div	cx			; divide number by 10
	add	dl,'0'			; make remainder ascii
	mov	[bx],dl			; and store in buffer
	or	ax,ax			; number zero?
	jz	FD_EXIT			; then finished
	dec	bx			; else, decrement the pointer
	jmp	FD_LOOP			; and do next digit
FD_EXIT:
	mov	dx,bx			; return beginning pointer in DX

	pop	cx
	pop	bx
	pop	ax
	ret

FMT_DECIMAL	endp

    page
;----------------------------------------------------------------------------
; Get a key from standard input, read next char. if entry is a function key
; to clear input buffer.

GET_KEY	proc near

	mov	ah,6			; dir. console i/o
	mov	dl,0FFH			; input
	int	DOSINT			;
	jz	GET_KEY			; loop while no char. avail.

	or	al,al			; function key?
	jnz	GK_EXIT			; no, exit
	mov	ah,6			; else, clear the extended code
	mov	dl,0FFH			;
	int	DOSINT			;
GK_EXIT:
	ret

GET_KEY	endp

;----------------------------------------------------------------------------

CODE	ends
end	START
	ret

GET_KEY	